@@ -24,7 +24,7 @@
 #include "DasherButtons.h"
 #include "DynamicFilter.h"
 #include "EyetrackerFilter.h"
-#include "OneButtonFilter.h"
+//#include "OneButtonFilter.h"
 #include "OneDimensionalFilter.h"
 #include "StylusFilter.h"
 #include "TwoButtonDynamicFilter.h"
@@ -156,8 +156,6 @@ void CDasherInterfaceBase::PreSetNotify(int iParameter, const std::string &sNewV
   switch(iParameter) {
   case SP_ALPHABET_ID: 
     // Cycle the alphabet history
-    std::cout << "Preset: " << sNewValue << std::endl;
-
     if(GetStringParameter(SP_ALPHABET_ID) != sNewValue) {
       if(GetStringParameter(SP_ALPHABET_1) != sNewValue) {
 	if(GetStringParameter(SP_ALPHABET_2) != sNewValue) {
@@ -229,52 +227,12 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
     case SP_INPUT_DEVICE:
       CreateInput();
       break;
-  //   case BP_NUMBER_DIMENSIONS:
-//     case BP_EYETRACKER_MODE:
-//     case BP_KEY_CONTROL:
-//     case BP_BUTTONONESTATIC:
-//     case BP_BUTTONONEDYNAMIC:
-//     case BP_BUTTONMENU:
-//     case BP_BUTTONDIRECT:
-//     case BP_BUTTONFOURDIRECT:
-//     case BP_BUTTONALTERNATINGDIRECT:
-//     case BP_COMPASSMODE:
-//     case BP_CLICK_MODE:
-//       // Delibarate fallthrough.  
-//       // FIXME - Horrible mess below - should really use
-//       // SP_INPUT_FILTER directly for the sake of sanity
-//       if(GetBoolParameter(BP_NUMBER_DIMENSIONS))
-// 	SetLongParameter(LP_INPUT_FILTER, 4); // 1D mode
-//       else if(GetBoolParameter(BP_EYETRACKER_MODE))
-// 	SetLongParameter(LP_INPUT_FILTER, 5);//"Eyetracker Mode"
-//       else if(GetBoolParameter(BP_CLICK_MODE))
-// 	SetLongParameter(LP_INPUT_FILTER, 7);//"Click Mode"
-//       else if(GetBoolParameter(BP_KEY_CONTROL)) {
-// 	// Various button modes
-// 	if(GetBoolParameter(BP_BUTTONONESTATIC))
-// 	  SetLongParameter(LP_INPUT_FILTER, 8);//"One Button Static"
-// 	else if(GetBoolParameter(BP_BUTTONONEDYNAMIC))
-// 	  SetLongParameter(LP_INPUT_FILTER, 6);//"One Button Dynamic"
-// 	else if(GetBoolParameter(BP_BUTTONMENU))
-// 	  SetLongParameter(LP_INPUT_FILTER, 8);//"Button Menu"
-// 	else if(GetBoolParameter(BP_BUTTONDIRECT))
-// 	  SetLongParameter(LP_INPUT_FILTER, 10); //"Three Button Direct"
-// 	else if(GetBoolParameter(BP_BUTTONFOURDIRECT))
-// 	  SetLongParameter(LP_INPUT_FILTER, 11);//"Four Button Direct"
-// 	else if(GetBoolParameter(BP_BUTTONALTERNATINGDIRECT))
-// 	  SetLongParameter(LP_INPUT_FILTER, 12);//"Alternating Direct"
-// 	else if(GetBoolParameter(BP_COMPASSMODE))
-// 	  SetLongParameter(LP_INPUT_FILTER, 13);//"Compass Mode"
-//       }
-//       else
-// 	SetLongParameter(LP_INPUT_FILTER, 3);//"Default"
-//       break;
     case SP_INPUT_FILTER:
       CreateInputFilter();
       break;
     default:
-      break;}
-      
+      break;
+    }
   }
   else if(pEvent->m_iEventType == 2) {
     CEditEvent *pEditEvent(static_cast < CEditEvent * >(pEvent));
@@ -322,10 +280,6 @@ void CDasherInterfaceBase::WriteTrainFilePartial() {
   strTrainfileBuffer = strTrainfileBuffer.substr(100);
 }
 
-// void CDasherInterfaceBase::RequestFullRedraw() {
-//   SetBoolParameter( BP_REDRAW, true );
-// }
-
 void CDasherInterfaceBase::CreateDasherModel() 
 {
   if(!m_AlphIO)
@@ -349,7 +303,13 @@ void CDasherInterfaceBase::CreateDasherModel()
       m_pDasherModel = 0;
     }
 
-    m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO);
+    if(m_deGameModeStrings.size() == 0)
+      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO);
+    else {
+      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO, true, m_deGameModeStrings[0]);
+      //      m_deGameModeStrings.pop_front();
+    }
+
     m_Alphabet = m_pDasherModel->GetAlphabetNew();
     
     string T = m_Alphabet->GetTrainingFile();
@@ -436,8 +396,6 @@ void CDasherInterfaceBase::Unpause(unsigned long Time) {
 
 void CDasherInterfaceBase::CreateInput() {
 
-  std::cout << "Creating input device: " << GetStringParameter(SP_INPUT_DEVICE) << std::endl;
-
   // FIXME - this shouldn't be the model used here - we should just change a parameter and work from the appropriate listener
 
   if(m_pInput) {
@@ -471,8 +429,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
 	int iNumDeleted = 0;
 	
 	if(m_pInputFilter) {
-	  bChanged = true; // TODO: Actually make CInputFilter::Timer return bool
-	  m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel); // FIXME - need logging stuff here
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel); // FIXME - need logging stuff here
 	}
 	
 	if (iNumDeleted > 0)
@@ -483,8 +440,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
       }
       else {
 	if(m_pInputFilter) {
-	  bChanged = true;
-	  m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel);
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel);
 	}
       }
       
@@ -494,6 +450,8 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
 
   Redraw(bChanged);
 
+  // This just passes the time through to the framerate tracker, so we
+  // know how often new frames are being drawn.
   if(m_pDasherModel != 0)
     m_pDasherModel->NewFrame(iTime);
 }
@@ -508,7 +466,7 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   
   if(bRedrawNodes) {
     m_pDasherView->Screen()->SendMarker(0);
-    m_pDasherModel->RenderToView(m_pDasherView,true);
+    m_pDasherModel->RenderToView(m_pDasherView, true);
   }
   
   m_pDasherView->Screen()->SendMarker(1);
@@ -516,8 +474,7 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   bool bDecorationsChanged(false);
   
   if(m_pInputFilter) {
-    bDecorationsChanged = true;
-    m_pInputFilter->DecorateView(m_pDasherView);
+    bDecorationsChanged = m_pInputFilter->DecorateView(m_pDasherView);
   }
   
   if(bRedrawNodes || bDecorationsChanged)
@@ -527,9 +484,9 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
 void CDasherInterfaceBase::ChangeAlphabet() {
 
   if(GetStringParameter(SP_ALPHABET_ID) == "") {
-    std::cout << "Looking up default alphabet" << std::endl;
-    
     SetStringParameter(SP_ALPHABET_ID, m_AlphIO->GetDefault());
+    // This will result in ChangeAlphabet() being called again, so
+    // exit from the first recursion
     return;
   }
   
@@ -773,6 +730,8 @@ void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
      while( m_pDasherModel->CheckForNewRoot(m_pDasherView) ) {
        // Do nothing
      }
+
+   Redraw(true);
 }
 
 
@@ -825,7 +784,7 @@ void CDasherInterfaceBase::ResetParameter(int iParameter) {
 
 // We need to be able to get at the UserLog object from outside the interface
 CUserLog* CDasherInterfaceBase::GetUserLogPtr() {
-	return m_pUserLog;
+  return m_pUserLog;
 }
 
 void CDasherInterfaceBase::KeyDown(int iTime, int iId) {
@@ -843,14 +802,17 @@ void CDasherInterfaceBase::KeyUp(int iTime, int iId) {
 void CDasherInterfaceBase::CreateInputFilter()
 {
   if(m_pInputFilter) {
+    m_pInputFilter->Deactivate();
     m_pInputFilter->Unref();
     m_pInputFilter = NULL;
   }
 
   m_pInputFilter = (CInputFilter *)GetModuleByName(GetStringParameter(SP_INPUT_FILTER));
 
-  if(m_pInputFilter)
+  if(m_pInputFilter) {
     m_pInputFilter->Ref();
+    m_pInputFilter->Activate();
+  }
 }
 
 void CDasherInterfaceBase::RegisterFactory(CModuleFactory *pFactory) {
@@ -872,7 +834,7 @@ void CDasherInterfaceBase::CreateFactories() {
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CEyetrackerFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CClickFilter(m_pEventHandler, m_pSettingsStore, this)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDynamicFilter(m_pEventHandler, m_pSettingsStore, this)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this, 14, 1, "Two Button Dynamic Mode")));
   // TODO: specialist factory for button mode
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 5, 1, true,8, "Menu Mode")));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 0, false,10, "Direct Mode")));
@@ -904,3 +866,7 @@ void CDasherInterfaceBase::GetPermittedValues(int iParameter, std::vector<std::s
 void CDasherInterfaceBase::StartShutdown() {
   m_bShutdownLock = true;
 }
+
+bool CDasherInterfaceBase::GetModuleSettings(const std::string &strName, SModuleSettings **pSettings, int *iCount) {
+  return GetModuleByName(strName)->GetSettings(pSettings, iCount);
+}
