@@ -127,3 +127,12 @@ double CDasherNode::GetProb(int iNormalization) {
   return (double) (m_iHbnd - m_iLbnd) / (double) iNormalization;
 }
 
+void CDasherNode::ConvertWithAncestors() {
+  if(m_bConverted)
+    return;
+  
+  m_bConverted = true;
+
+  if(m_pParent)
+    m_pParent->ConvertWithAncestors();
+}
