@@ -21,6 +21,7 @@
 #include "dasher_internal_buffer.h"
 #include "dasher_lock_dialogue.h"
 #include "fileops.h"
+#include "game_mode_helper.h"
 #include "FontDialogues.h"
 #include "GtkDasherControl.h"
 #include "Preferences.h"
@@ -48,6 +49,10 @@
 
 // ---
 
+#define ACTION_STATE_SHOW 1
+#define ACTION_STATE_CONTROL 2
+#define ACTION_STATE_AUTO 4
+
 typedef struct _EditorAction EditorAction;
 
 struct _EditorAction {
@@ -74,6 +79,7 @@ void dasher_editor_display_message(DasherEditor *pSelf, DasherMessageInfo *pMess
 void dasher_editor_handle_parameter_change(DasherEditor *pSelf, int iParameter);
 void dasher_editor_handle_pre_parameter_change(DasherEditor *pSelf, int iParameter);
 void dasher_editor_check_activity(DasherEditor *pSelf, EditorAction *pAction);
+void dasher_editor_action_save_state(DasherEditor *pSelf, EditorAction *pAction);
 
 // TODO: Should these be public?
 void dasher_editor_convert(DasherEditor *pSelf);
@@ -82,6 +88,7 @@ void dasher_editor_protect(DasherEditor *pSelf);
 // Private methods not in class
 extern "C" void action_button_callback(GtkWidget *pWidget, gpointer pUserData);
 extern "C" void context_changed_handler(GObject *pSource, gpointer pUserData);
+extern "C" void main_window_realized(DasherMain *pMain, gpointer pUserData);
 
 typedef struct _DasherEditorPrivate DasherEditorPrivate;
 
@@ -96,7 +103,9 @@ struct _DasherEditorPrivate {
   gboolean bActionIterStarted;
   gint iNextActionID;
   IDasherBufferSet *pBufferSet;
-  int iConversionCount;
+  IDasherBufferSet *pExternalBuffer;
+  IDasherBufferSet *pInternalBuffer;
+  GameModeHelper *pGameModeHelper;
 };
 
 GType dasher_editor_get_type() {
@@ -131,6 +140,8 @@ static void dasher_editor_init(DasherEditor *pDasherControl) {
   pDasherControl->private_data = new DasherEditorPrivate;
 
   ((DasherEditorPrivate *)(pDasherControl->private_data))->pBufferSet = 0;
+  ((DasherEditorPrivate *)(pDasherControl->private_data))->pInternalBuffer = 0;
+  ((DasherEditorPrivate *)(pDasherControl->private_data))->pExternalBuffer = 0;
 }
 
 static void dasher_editor_destroy(GObject *pObject) {
@@ -162,6 +173,7 @@ DasherEditor *dasher_editor_new(int argc, char **argv) {
   g_pDasherAppSettings = dasher_app_settings_new(argc, argv);
 
   g_pDasherMain = dasher_main_new();
+  //  g_signal_connect(G_OBJECT(g_pDasherMain), "realized", G_CALLBACK(main_window_realized), pDasherControl);
 
   GladeXML *pGladeXML = dasher_main_get_glade(g_pDasherMain);
   pDasherWidget = glade_xml_get_widget(pGladeXML, "DasherControl");
@@ -169,7 +181,11 @@ DasherEditor *dasher_editor_new(int argc, char **argv) {
   dasher_main_set_app_settings(g_pDasherMain, g_pDasherAppSettings);
 
   // TODO: Make lock diaogue a full method
+#ifndef WITH_MAEMO
   dasher_lock_dialogue_new(pGladeXML, GTK_WINDOW(dasher_main_get_window(g_pDasherMain)));
+#else
+  dasher_lock_dialogue_new(pGladeXML, 0);
+#endif
   // TODO: Bring into object framework
   InitialiseFontDialogues(pGladeXML);
   
@@ -185,13 +201,12 @@ DasherEditor *dasher_editor_new(int argc, char **argv) {
   pPrivate->pPrimarySelection = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
   pPrivate->pActionRing = NULL;
   pPrivate->iNextActionID = 0;
-  pPrivate->iConversionCount = 0;
-
-  dasher_editor_setup_actions(pDasherControl);
-  dasher_preferences_dialogue_populate_actions(g_pPreferencesDialogue);
+  pPrivate->pGameModeHelper = 0;
 
   dasher_editor_create_buffer(pDasherControl);
-
+  dasher_editor_setup_actions(pDasherControl);
+  dasher_preferences_dialogue_populate_actions(g_pPreferencesDialogue);
+ 
   return pDasherControl;
 }
 
@@ -363,19 +378,28 @@ void dasher_editor_add_action(DasherEditor *pSelf, DasherAction *pNewAction) {
   pNewEditorAction->iID = pPrivate->iNextActionID;
   ++pPrivate->iNextActionID;
 
-  // TODO: Need to get/set registry keys for these
-  // TODO: Currently hack things so speech is off by default
+  gchar szRegistryName[256];
+  strncpy(szRegistryName, "Action_", 256);
+  strncat(szRegistryName, dasher_action_get_name(pNewEditorAction->pAction), 255 - strlen(szRegistryName));
 
-  if(!strcmp(dasher_action_get_name(pNewEditorAction->pAction), "Speak")) {
-    pNewEditorAction->bShow = false;
-    pNewEditorAction->bControl = false;
-  }
-  else {
-    pNewEditorAction->bShow = true;
-    pNewEditorAction->bControl = true;
+  for(int i(0); i < strlen(szRegistryName); ++i)
+    if(szRegistryName[i] == ' ')
+      szRegistryName[i] = '_';
+
+  gint iState;
+
+  if(!dasher_app_settings_get_free_long(g_pDasherAppSettings, szRegistryName, iState)) {
+    if(!strcmp(dasher_action_get_name(pNewEditorAction->pAction), "Speak"))
+      iState = 0;
+    else
+      iState = ACTION_STATE_SHOW | ACTION_STATE_CONTROL;
+
+    dasher_app_settings_set_free_long(g_pDasherAppSettings, szRegistryName, iState);
   }
 
-  pNewEditorAction->bAuto = false;
+  pNewEditorAction->bShow = iState & ACTION_STATE_SHOW;
+  pNewEditorAction->bControl = iState & ACTION_STATE_CONTROL;
+  pNewEditorAction->bAuto = iState & ACTION_STATE_AUTO;
 
   dasher_editor_check_activity(pSelf, pNewEditorAction);
 
@@ -404,7 +428,7 @@ void dasher_editor_setup_actions(DasherEditor *pSelf) {
 #endif
 
 #ifdef GNOME_A11Y
-  dasher_editor_add_action(pSelf, DASHER_ACTION(dasher_action_keyboard_new()));
+  dasher_editor_add_action(pSelf, DASHER_ACTION(dasher_action_keyboard_new(pPrivate->pExternalBuffer)));
 #endif
 
 #ifdef WITH_MAEMO
@@ -472,13 +496,19 @@ void dasher_editor_setup_actions(DasherEditor *pSelf) {
     pCurrentAction = pCurrentAction->pNext;
   }
 
+#ifndef WITH_MAEMOFULLSCREEN
   dasher_editor_rebuild_action_pane(pSelf);
+#endif
 }
 
 extern "C" void delete_children_callback(GtkWidget *pWidget, gpointer pUserData) {
   gtk_widget_destroy(pWidget);
 }
 
+extern "C" void main_window_realized(DasherMain *pMain, gpointer pUserData) {
+}
+
+
 void dasher_editor_rebuild_action_pane(DasherEditor *pSelf) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
   
@@ -592,6 +622,8 @@ void dasher_editor_action_set_show(DasherEditor *pSelf, int iActionID, bool bVal
     pAction->bShow = bValue;
     dasher_editor_check_activity(pSelf, pAction);
     dasher_editor_rebuild_action_pane(pSelf);
+
+    dasher_editor_action_save_state(pSelf, pAction);
   }
 }
 
@@ -608,6 +640,8 @@ void dasher_editor_action_set_control(DasherEditor *pSelf, int iActionID, bool b
       gtk_dasher_control_connect_node(GTK_DASHER_CONTROL(pDasherWidget), pAction->iControlID, Dasher::CControlManager::CTL_USER, -2);
     else
       gtk_dasher_control_disconnect_node(GTK_DASHER_CONTROL(pDasherWidget), pAction->iControlID, Dasher::CControlManager::CTL_USER);
+    
+    dasher_editor_action_save_state(pSelf, pAction);
   }
 }
 
@@ -618,29 +652,58 @@ EditorAction *pAction;
   if(pAction) {
     pAction->bAuto = bValue;
     dasher_editor_check_activity(pSelf, pAction);
+    
+    dasher_editor_action_save_state(pSelf, pAction);
   }
 }
 
+void dasher_editor_action_save_state(DasherEditor *pSelf, EditorAction *pAction) {
+  gchar szRegistryName[256];
+  strncpy(szRegistryName, "Action_", 256);
+  strncat(szRegistryName, dasher_action_get_name(pAction->pAction), 255 - strlen(szRegistryName));
+
+  for(int i(0); i < strlen(szRegistryName); ++i)
+    if(szRegistryName[i] == ' ')
+      szRegistryName[i] = '_';
+
+  gint iState = 0;
+
+  if(pAction->bShow)
+    iState += ACTION_STATE_SHOW;
+  
+  if(pAction->bControl)
+    iState += ACTION_STATE_CONTROL;
+
+  if(pAction->bAuto)
+    iState += ACTION_STATE_AUTO;
+
+  dasher_app_settings_set_free_long(g_pDasherAppSettings, szRegistryName, iState);
+}
+
 void dasher_editor_output(DasherEditor *pSelf, const gchar *szText) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
   idasher_buffer_set_insert(pPrivate->pBufferSet, szText);
-  ++pPrivate->iConversionCount;
+
+  if(pPrivate->pGameModeHelper)
+    game_mode_helper_output(pPrivate->pGameModeHelper, szText);
 }
 
 void dasher_editor_delete(DasherEditor *pSelf, int iLength) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
   idasher_buffer_set_delete(pPrivate->pBufferSet, iLength);
+
+  if(pPrivate->pGameModeHelper)
+    game_mode_helper_delete(pPrivate->pGameModeHelper, iLength);
 }
 
 void dasher_editor_convert(DasherEditor *pSelf) {
- DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
- idasher_buffer_set_delete(pPrivate->pBufferSet, pPrivate->iConversionCount);
+  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
+  idasher_buffer_set_edit_convert(pPrivate->pBufferSet);
 }
 
 void dasher_editor_protect(DasherEditor *pSelf) {
- DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
-
- pPrivate->iConversionCount = 0;
+  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
+  idasher_buffer_set_edit_protect(pPrivate->pBufferSet);
 }
 
 EditorAction *dasher_editor_get_action_by_id(DasherEditor *pSelf, int iID){
@@ -662,19 +725,23 @@ EditorAction *dasher_editor_get_action_by_id(DasherEditor *pSelf, int iID){
 void dasher_editor_create_buffer(DasherEditor *pSelf) {
   DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
 
-  if(pPrivate->pBufferSet != 0) {
-    g_object_unref(G_OBJECT(pPrivate->pBufferSet));
+#ifdef GNOME_A11Y
+  if(!(pPrivate->pExternalBuffer))
+    pPrivate->pExternalBuffer = IDASHER_BUFFER_SET(dasher_external_buffer_new());
+#endif
+
+  if(!(pPrivate->pInternalBuffer)) {
+    pPrivate->pInternalBuffer = dasher_editor_get_buffer_set(pSelf);
+    idasher_buffer_set_conversion_mode(pPrivate->pInternalBuffer, gtk_dasher_control_get_parameter_bool(GTK_DASHER_CONTROL(pDasherWidget), BP_CONVERSION_MODE));
   }
   
   if(dasher_app_settings_get_long(g_pDasherAppSettings, APP_LP_STYLE) == 2)
-#ifdef GNOME_A11Y
-    pPrivate->pBufferSet = IDASHER_BUFFER_SET(dasher_external_buffer_new());
-#else
-  pPrivate->pBufferSet = 0;
-#endif
-  else
-    pPrivate->pBufferSet = dasher_editor_get_buffer_set(pSelf);
-  
+    pPrivate->pBufferSet = pPrivate->pExternalBuffer;
+  else {
+    pPrivate->pBufferSet = pPrivate->pInternalBuffer;
+  }
+
+  // TODO: Fix this
   g_signal_connect(G_OBJECT(pPrivate->pBufferSet), "context_changed", G_CALLBACK(context_changed_handler), NULL);
 }
 
@@ -746,6 +813,12 @@ void dasher_editor_check_activity(DasherEditor *pSelf, EditorAction *pAction) {
     dasher_action_deactivate(pAction->pAction);
 }
 
+void dasher_editor_start_tutorial(DasherEditor *pSelf) {
+  DasherEditorPrivate *pPrivate = (DasherEditorPrivate *)(pSelf->private_data);
+
+  pPrivate->pGameModeHelper = GAME_MODE_HELPER(game_mode_helper_new(GTK_DASHER_CONTROL(pDasherWidget)));
+}
+
 // Callbacks
 
 extern "C" void action_button_callback(GtkWidget *pWidget, gpointer pUserData) { 
