@@ -1,7 +1,12 @@
+#include "../../config.h"
+
 #include "GtkDasherControl.h"
 #include "dasher.h"
 #include "dasher_lock_dialogue.h"
 
+#ifdef WITH_MAEMO
+#include <hildon-widgets/hildon-banner.h>
+#endif
 #include <iostream>
 
 // TODO: Make this a real class
@@ -11,6 +16,7 @@ GtkWidget *m_pLockProgress;
 GtkWidget *m_pLockMessage;
 
 void dasher_lock_dialogue_new(GladeXML *pGladeXML, GtkWindow *pMainWindow) {
+#ifndef WITH_MAEMO
   m_pLockWindow = glade_xml_get_widget(pGladeXML, "lock_window");
   m_pLockProgress = glade_xml_get_widget(pGladeXML, "lock_progress");
   m_pLockMessage = glade_xml_get_widget(pGladeXML, "lock_message");
@@ -18,6 +24,9 @@ void dasher_lock_dialogue_new(GladeXML *pGladeXML, GtkWindow *pMainWindow) {
   gtk_widget_hide(m_pLockWindow);
   
   gtk_window_set_transient_for(GTK_WINDOW(m_pLockWindow), pMainWindow);
+#else
+  m_pLockWindow = 0;
+#endif
 }
 
 extern "C" void on_lock_info(GtkDasherControl *pDasherControl, gpointer pLockInfo, gpointer pUserData) {
@@ -27,6 +36,7 @@ extern "C" void on_lock_info(GtkDasherControl *pDasherControl, gpointer pLockInf
 
   DasherLockInfo *pInfo = (DasherLockInfo *)pLockInfo;
 
+#ifndef WITH_MAEMO
   gtk_label_set_text(GTK_LABEL(m_pLockMessage), pInfo->szMessage);
   gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(m_pLockProgress), pInfo->iPercent / 100.0);
 
@@ -34,6 +44,19 @@ extern "C" void on_lock_info(GtkDasherControl *pDasherControl, gpointer pLockInf
     gtk_widget_show(m_pLockWindow);
   else
     gtk_widget_hide(m_pLockWindow);
+#else
+  if(pInfo->bLock) {
+    if(!m_pLockWindow)
+      m_pLockWindow = hildon_banner_show_progress(NULL, NULL, pInfo->szMessage);
+
+    hildon_banner_set_fraction(HILDON_BANNER(m_pLockWindow), pInfo->iPercent / 100.0);
+  }
+  else {
+    if(m_pLockWindow)
+      gtk_widget_destroy(m_pLockWindow);
+    m_pLockWindow = 0;
+  }
+#endif
 
   // Keep the GTK interface responsive
   while(gtk_events_pending())
